/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - October 4, 2003.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

var gBrowser = null; // declared in navigator.js but out of reach for us!
const nsIWindowDataSource = Components.interfaces.nsIWindowDataSource; // We need this for multiviewsRDF.js!
const nsIRDFResource = Components.interfaces.nsIRDFResource;
const nsIRDFLiteral = Components.interfaces.nsIRDFLiteral;
////////////////////////////////////////////////////////////////////////////////
function getLastActiveBrowserWindow()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var window = windowManagerInterface.getMostRecentWindow("navigator:browser");
  return window;
}
////////////////////////////////////////////////////////////////////////////////
function loadURI(url)
{
  try {
    getWebNavigation().loadURI(url, nsIWebNavigation.LOAD_FLAGS_NONE, referrer, null, null);
  } catch (ex) {
  }
}
////////////////////////////////////////////////////////////////////////////////
function initContextMenu(aEvent)
{
  var rows = tree.treeBoxObject.view.rowCount;
  var selectedResources = sessionManager.getSelectedResources();

  // Disable "Select All" when there's a selection
  var selectAll = document.getElementById("selectAll");
  // Disable "Deselect All" when there's no selection
  var deselectAll = document.getElementById("undoSelection");
  // Disable 'Reload Tab Every...' menu item when multiple items are selected
  var reloadMenuitem = document.getElementById("");

  if (selectedResources.length > 1) {
    selectAll.setAttribute("disabled", "true");
    deselectAll.removeAttribute("disabled");
  }
  else { // No selection, enable "Select All" and disable "Deselect All"
    if (rows > 1)
      selectAll.removeAttribute("disabled");
    deselectAll.setAttribute("disabled", "true");
  }
  var isWindow = tree.treeBoxObject.view.isContainer(sessionManager.getRowNumber(aEvent)) ? false : true;
  var tabElements = document.getElementsByAttribute("display_for", "tab");
  var windowElements = document.getElementsByAttribute("display_for", "window");
  var tabAndWindowElements = document.getElementsByAttribute("display_for", "tab-and-window");
  var isMixed = sessionManager.checkForMixedSelection();

  for (var i = 0; i < tabElements.length; i++)
    tabElements[i].setAttribute("disabled", !isWindow);
  for (i = 0; i < windowElements.length; i++)
    windowElements[i].setAttribute("disabled", isWindow);
  for (i = 0; i < tabAndWindowElements.length; i++)
    tabAndWindowElements[i].setAttribute("disabled", isMixed ? "false" : "true");

  // Disable "Reload Tab Every..." when multiple items are selected
  var reloadTab = document.getElementById("set-interval");

  if (selectedResources.length > 1)
    reloadTab.setAttribute("disabled", "true");
}
////////////////////////////////////////////////////////////////////////////////
// This function is copied over from multiviewsInit.js because it is not 
// available from multiviewsSessionManager.xul (read 'it is not included').
////////////////////////////////////////////////////////////////////////////////
function getNumberOfWindowsByType(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowCounter = 0;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManager.getEnumerator(aType);

  while (enumerator.hasMoreElements()) {
    var aWindow = enumerator.getNext();
    windowCounter++;
  }
  return windowCounter;
}
////////////////////////////////////////////////////////////////////////////////
var sessionManager = 
{
  tree : null,
  tabbrowser : null,
  lastResource : null,
  reloadInterval : null,
  currentRow : null,

  tooltipBox : null,
  tabNumberTooltip : null,
  sessionNumberTooltip : null,
  windowNumberTooltip : null,

  stringBundle : null,

  initSessionManager: function()
  {
    tree = document.getElementById("sessionManagerTree");
    initializeRDFStorageFile();
    tree.database.AddDataSource(mzRDFStore);
    tree.builder.rebuild();
    tabbrowser = getLastActiveBrowserWindow().getBrowser();
    stringBundle = document.getElementById("properties");
    tooltipBox = document.getElementById("tooltipBox");
    tabNumberTooltip = tooltipBox.childNodes[0];
    sessionNumberTooltip = tooltipBox.childNodes[1];
    windowNumberTooltip = tooltipBox.childNodes[2];
    lastResource = null;
    reloadInterval = 0;
    currentRow = -1;

    // No need to open a useless Tab Session Manager!
    if (tree.treeBoxObject.view.rowCount == 0) {
      /* var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
      promptService.alert(window, mzGetTextFromBundle("mzWizard"), mzGetTextFromBundle("noTabSessionAvailable")); */

      alert("You need to save a browser session, before you can use the Tab Session Manager");
      window.close();
    }
  },

  reInitSessionManager: function()
  {
    tree = document.getElementById("sessionManagerTree");
    tree.builder.rebuild();
    lastResource = null;
  },

  click: function(aEvent)
  {
    currentRow = this.getRowNumber(aEvent);
    reloadInterval = sessionManager.getReloadInterval(aEvent);

    if (aEvent.button == 1 || (aEvent.button == 0 && aEvent.detail == 2)) {
      var resource = tree.view.getResourceAtIndex(tree.currentIndex).Value;
      if (tree.treeBoxObject.view.isContainer(this.getRowNumber(aEvent))) {
        if (aEvent.button == 1) {
          lastResource = resource;
          restoreTabsFromResource(tabbrowser, resource);
        }
      }
      else {
        lastResource = resource;
        restoreTabFromResource(tabbrowser, RDF.GetResource(resource), true);
      }
    }
  },

  getRowNumber: function(aEvent)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    return tree.treeBoxObject.getRowAt(X, Y);
  },

  getResourceForRow: function(aRowNumber)
  {
     return tree.view.getResourceAtIndex(aRowNumber).Value;
  },

  getResourceSpecs: function(aResource)
  {
    aResource = aResource.replace("urn:session-", "");
    aResource = aResource.replace("window-", "");
    aResource = aResource.replace("tabs:root:tab-", "");
    return aResource.split(":");
  },

  convertResourceSpecsToHumanData: function(aDataArray)
  {
     for (i in aDataArray)
       aDataArray[i] = Number(aDataArray[i])+1;
     return aDataArray;
  },

  getSelectedResources: function()
  {
    var rows = -1;
    var resources = new Array();
    // Get and return all selection ranges
    var rangeCount = tree.treeBoxObject.selection.getRangeCount();
    // Itterate over all ranges to collect the selected rows
    for (var range = 0; range < rangeCount; range++) {
      var startSelection = {};
      var endSelection = {};
      tree.treeBoxObject.selection.getRangeAt(range, startSelection, endSelection);
      // Collect all items and store them in our resource array
      for (var row = startSelection.value; row <= endSelection.value; row++) {
        resources[++rows] = this.getResourceForRow(row);
      }
    }
    if (resources)
      return resources;
    return null;
  },

  getSelectedRows: function()
  {
    var rows = -1;
    var selectedRows = new Array();
    // Get and return all selection ranges
    var rangeCount = tree.treeBoxObject.selection.getRangeCount();
    // Itterate over all ranges to collect the selected rows
    for (var range = 0; range < rangeCount; range++) {
      var startSelection = {};
      var endSelection = {};
      tree.treeBoxObject.selection.getRangeAt(range, startSelection, endSelection);
      // Collect all rows and store them in our array
      for (var row = startSelection.value; row <= endSelection.value; row++) {
        selectedRows[++rows] = row;
      }
    }
    if (selectedRows)
      return selectedRows;
    return null;
  },


  checkForMixedSelection: function()
  {
    var tabs = 0;
    var windows = 0;
    var mixed = false;

    // Collect selected tree rows
    var selectedItems = this.getSelectedRows();
    // Do we have any selected items?
    if (selectedItems && selectedItems.length > 0) {
      // Itterate over the seleted items
      for (var row = 0; row < selectedItems.length; row++) {
        if (tree.treeBoxObject.view.isContainer(selectedItems[row]))
          windows++;
        else
          tabs++;
        // Do we have a mixed selection?
        if (tabs > 0 && windows > 0) {
          mixed = true;
          break;
        }
      }
    }
    return mixed;
  },

  removeWindowItemsFromSelection: function()
  {
    // Collect selected tree rows
    var selectedItems = this.getSelectedRows();
    // Do we have any selected items?
    if (selectedItems && selectedItems.length > 0) {
      // Itterate over the seleted items to clear the window items
      for (var row = 0; row < selectedItems.length; row++) {
        if (tree.treeBoxObject.view.isContainer(row)) {
          tree.treeBoxObject.selection.clearRange(row, row);
        }
      }
    }
  },

  getReloadInterval: function(aEvent)
  {
    var row = this.getRowNumber(aEvent);
    var interval = tree.view.getCellText(row, "sm-interval");
    return interval || 0;
  },

  setReloadInterval: function(aEvent, aValue)
  {
    var resource = this.getResourceForRow(currentRow);
    var intervalProp = RDF.GetResource("http://multizilla.mozdev.org/rdf#interval");
    var currentValue = mzRDFStore.GetTarget(RDF.GetResource(resource), intervalProp, true);

    mzRDFStore.Change(RDF.GetResource(resource), intervalProp, currentValue, RDF.GetLiteral(aValue));
    mzRDFStore.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  },

  setRowTooltip: function(aEvent)
  {
    var row = this.getRowNumber(aEvent);
    var resource = this.getResourceForRow(row);
    var resourceValues = this.getResourceSpecs(resource);
    resourceValues = this.convertResourceSpecsToHumanData(resourceValues);

    if (resourceValues.length == 3) {
      tabNumberTooltip.setAttribute("value", "Tab: " + resourceValues[2]);
      tabNumberTooltip.removeAttribute("hidden");
    }
    else
      tabNumberTooltip.setAttribute("hidden", "true");

    windowNumberTooltip.setAttribute("value", "Window: " + resourceValues[1]);
    sessionNumberTooltip.setAttribute("value", "Session: " + resourceValues[0]);
  },

  doSelect: function(aCommand)
  {
    switch (aCommand) {
      case "all": // Called from the context menu, or Ctrl+A ?
           tree.treeBoxObject.selection.selectAll();
           this.removeWindowItemsFromSelection();
           break;
      case "none": // Called from the context menu, or Ctrl+Z ?
           tree.treeBoxObject.selection.clearSelection();
           tree.treeBoxObject.selection.select(tree.currentIndex);
           break;
    }
    tree.focus();
  },

  doConfirm: function(additionalText)
  {
    var doAction = false;
    var dialogTitle = stringBundle.getString("tsmConfirmActionTitle");
    var dialogText = stringBundle.getString("tsmConfirmActionText");
    additionalText = stringBundle.getString(additionalText);
    dialogText = dialogText.replace(/%S/, additionalText);
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService();

    promptService = promptService.QueryInterface(Components.interfaces.nsIPromptService);
    doAction = promptService.confirm(window, dialogTitle, dialogText);
    tree.focus();
    return doAction;
  },

  onRemove: function(skipLastResourceCheck)
  {
    var rows = this.getSelectedRows();
    var confirmActionText = (rows > 1) ? "tsmRemoveSingleItem" : "tsmRemoveMultipleItems";

    if (!this.doConfirm(confirmActionText))
      return;

    if (rows && rows.length) {
      for (i in rows) {
        /***
          * "urn:session-x:window-x:tabs:root" for windows
          * "urn:session-x:window-x:tabs:tab-x" for tabs
          */
        var resource = this.getResourceForRow(rows[i]);
        // Prevent duplicated actions!
        if (!skipLastResourceCheck && (lastResource == null || lastResource == resource))
          break;
        if (tree.treeBoxObject.view.isContainer(rows[i])) { // Window
          removeAllResourcesFor(resource);
        }
        else { // Single Tab
          removeResources(resource);
        }
      }
    }
    this.reInitSessionManager();
  },

  onAccept: function(skipLastResourceCheck, closeSessionManager)
  {
    var rows = this.getSelectedRows();

    if (rows && rows.length) {
      for (i in rows) {
        var resource = this.getResourceForRow(rows[i]);
        // Prevent duplicated actions!
        if (!skipLastResourceCheck && (/* lastResource == null || */ lastResource == resource)) {
          continue;
        }
        if (tree.treeBoxObject.view.isContainer(rows[i])) { // Window
          var browser = openWindowAndReturnBrowser();
          restoreTabsFromResource(browser, resource, false);
          closeSessionManager = true;
        }
        else { // Single Tab
          restoreTabFromResource(tabbrowser, RDF.GetResource(resource), true, false);
        }
      }
    }
    if (closeSessionManager)
      window.close();
  },

  onCancel: function()
  {
    window.close();
  },
  
  onHelp: function()
  {
    var helpPages = stringBundle.getString("helpPages-SessionManager");
    // We could eventually check the on-line status!
    tabbrowser.loadURI(helpPages);
    window.close();
  }

};

addEventListener("load", sessionManager.initSessionManager, false);